@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2010  Alexandre Courbot
+ *  Copyright (C) 2010-2012  Alexandre Courbot
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,11 +16,13 @@
  */
 
 #include "core/Paths.h"
+#include <core/Database.h>
 #include "gui/EntriesViewHelper.h"
 #include "gui/EntryMenu.h"
 #include "gui/EditEntryNotesDialog.h"
 #include "gui/TagsDialogs.h"
 #include "gui/EntriesPrinter.h"
+#include "gui/BatchHandler.h"
 
 #include <QAction>
 #include <QProgressDialog>
@@ -89,93 +91,86 @@ void EntriesViewHelper::setPreferenceHandler(Preference pref, PreferenceRoot *re
 QList<EntryPointer> EntriesViewHelper::selectedEntries() const
 {
 	QModelIndexList selection = client()->selectionModel()->selectedIndexes();
+	QProgressDialog progressDialog(tr("Selecting entries..."), tr("Abort"), 0, selection.size(), 0);
+	progressDialog.setMinimumDuration(1000);
+	progressDialog.setWindowTitle(tr("Please wait..."));
+	progressDialog.setWindowModality(Qt::WindowModal);
+
 	QList<EntryPointer> selectedEntries;
+	int i = 0;
+	int completed = true;
 	foreach(const QModelIndex &index, selection) {
+		if (progressDialog.wasCanceled()) {
+			completed = false;
+			break;
+		}
+		progressDialog.setValue(i++);
 		EntryPointer entry(qVariantValue<EntryPointer>(index.data(Entry::EntryRole)));
 		if (entry) selectedEntries << entry;
 	}
-	return selectedEntries;
+	if (!completed) return QList<EntryPointer>();
+	else return selectedEntries;
 }
 
+void EntriesViewHelper::applyOnSelection(const BatchHandler &handler)
+{
+	QModelIndexList selection(client()->selectionModel()->selectedIndexes());
+
+	BatchHandler::applyOnEntries(handler, selectedEntries());
+	foreach (const QModelIndex &index, selection)
+		client()->update(index);
+}
+
+struct StudyHandler : BatchHandler
+{
+	void apply(const EntryPointer &e) const
+	{
+		e->addToTraining();
+	}
+};
+
 void EntriesViewHelper::studySelected()
 {
-	QModelIndexList selection = client()->selectionModel()->selectedIndexes();
-	// Progress bar
-	QProgressDialog progressDialog(tr("Marking entries..."), tr("Abort"), 0, selection.size(), client());
-	progressDialog.setMinimumDuration(1000);
-	progressDialog.setWindowTitle(tr("Operation in progress..."));
-	progressDialog.setWindowModality(Qt::WindowModal);
-
-	int i = 0;
-	foreach (const QModelIndex &index, selection) {
-		if (progressDialog.wasCanceled()) break;
-		progressDialog.setValue(i++);
-		EntryPointer entry = qVariantValue<EntryPointer>(index.data(Entry::EntryRole));
-		if (!entry) continue;
-		entry->addToTraining();
-		client()->update(index);
-	}
+	applyOnSelection(StudyHandler());
 }
 
+struct UnstudyHandler : BatchHandler
+{
+	void apply(const EntryPointer &e) const
+	{
+		e->removeFromTraining();
+	}
+};
+
 void EntriesViewHelper::unstudySelected()
 {
-	QModelIndexList selection = client()->selectionModel()->selectedIndexes();
-	// Progress bar
-	QProgressDialog progressDialog(tr("Marking entries..."), tr("Abort"), 0, selection.size(), client());
-	progressDialog.setMinimumDuration(1000);
-	progressDialog.setWindowTitle(tr("Operation in progress..."));
-	progressDialog.setWindowModality(Qt::WindowModal);
-
-	int i = 0;
-	foreach (const QModelIndex &index, selection) {
-		if (progressDialog.wasCanceled()) break;
-		progressDialog.setValue(i++);
-		EntryPointer entry = qVariantValue<EntryPointer>(index.data(Entry::EntryRole));
-		if (!entry) continue;
-		entry->removeFromTraining();
-		client()->update(index);
-	}
+	applyOnSelection(UnstudyHandler());
 }
 
+struct MarkAsKnownHandler : BatchHandler
+{
+	void apply(const EntryPointer &e) const
+	{
+		e->setAlreadyKnown();
+	}
+};
+
 void EntriesViewHelper::markAsKnown()
 {
-	QModelIndexList selection = client()->selectionModel()->selectedIndexes();
-	// Progress bar
-	QProgressDialog progressDialog(tr("Marking entries..."), tr("Abort"), 0, selection.size(), client());
-	progressDialog.setMinimumDuration(1000);
-	progressDialog.setWindowTitle(tr("Operation in progress..."));
-	progressDialog.setWindowModality(Qt::WindowModal);
-
-	int i = 0;
-	foreach (const QModelIndex &index, selection) {
-		if (progressDialog.wasCanceled()) break;
-		progressDialog.setValue(i++);
-		EntryPointer entry = qVariantValue<EntryPointer>(index.data(Entry::EntryRole));
-		if (!entry) continue;
-		if (entry->alreadyKnown()) continue;
-		entry->setAlreadyKnown();
-		client()->update(index);
-	}
+	applyOnSelection(MarkAsKnownHandler());
 }
 
+struct ResetTrainingHandler : BatchHandler
+{
+	void apply(const EntryPointer &e) const
+	{
+		e->resetScore();
+	}
+};
+
 void EntriesViewHelper::resetTraining()
 {
-	QModelIndexList selection = client()->selectionModel()->selectedIndexes();
-	// Progress bar
-	QProgressDialog progressDialog(tr("Resetting entries..."), tr("Abort"), 0, selection.size(), client());
-	progressDialog.setMinimumDuration(1000);
-	progressDialog.setWindowTitle(tr("Operation in progress..."));
-	progressDialog.setWindowModality(Qt::WindowModal);
-
-	int i = 0;
-	foreach (const QModelIndex &index, selection) {
-		if (progressDialog.wasCanceled()) break;
-		progressDialog.setValue(i++);
-		EntryPointer entry = qVariantValue<EntryPointer>(index.data(Entry::EntryRole));
-		if (!entry) continue;
-		entry->resetScore();
-		client()->update(index);
-	}
+	applyOnSelection(ResetTrainingHandler());
 }
 
 void EntriesViewHelper::setTags()
@@ -194,24 +189,20 @@ void EntriesViewHelper::addTags()
 		client()->update(index);
 }
 
+struct AddTagsListHandler : BatchHandler
+{
+	const QStringList &_tags;
+	AddTagsListHandler(const QStringList &tags) : _tags(tags) {}
+
+	void apply(const EntryPointer &e) const
+	{
+		e->addTags(_tags);
+	}
+};
+
 void EntriesViewHelper::addTags(const QStringList &tags)
 {
-	QModelIndexList selection = client()->selectionModel()->selectedIndexes();
-	// Progress bar
-	QProgressDialog progressDialog(tr("Adding tags..."), tr("Abort"), 0, selection.size(), client());
-	progressDialog.setMinimumDuration(1000);
-	progressDialog.setWindowTitle(tr("Operation in progress..."));
-	progressDialog.setWindowModality(Qt::WindowModal);
-	
-	int i = 0;
-	foreach (const QModelIndex &index, selection) {
-		if (progressDialog.wasCanceled()) break;
-		progressDialog.setValue(i++);
-		EntryPointer entry = qVariantValue<EntryPointer>(index.data(Entry::EntryRole));
-		if (!entry) continue;
-		entry->addTags(tags);
-		client()->update(index);
-	}
+	applyOnSelection(AddTagsListHandler(tags));
 }
 
 void EntriesViewHelper::addNote()
@@ -237,22 +228,44 @@ bool EntriesViewHelper::askForPrintOptions(QPrinter &printer, const QString &tit
 	return true;
 }
 
+static bool modelIndexLessThan(const QModelIndex &i1, const QModelIndex &i2)
+{
+	// Just in case we compare the same index - should not happen in real life
+	if (i1 == i2) return false;
+
+	// Build lists of the path that leads to both indexes from the root node
+	QList<QModelIndex> l1, l2;
+	QModelIndex p(i1);
+	do {
+		l1.prepend(p);
+		if (!p.isValid()) break;
+		p = p.parent();
+	} while (true);
+	p = i2;
+	do {
+		l2.prepend(p);
+		if (!p.isValid()) break;
+		p = p.parent();
+	} while (true);
+
+	// Check if one is the parent of another
+	if (l2.contains(i1)) return true;
+	if (l1.contains(i2)) return false;
+
+	// Otherwise find the common lowest ancestor
+	int lca = 0;
+	while (l1[lca] == l2[lca]) lca++;
+
+	// And check which child is lower than the other
+	return l1[lca].row() < l2[lca].row();
+}
+
 QModelIndexList EntriesViewHelper::getAllIndexes(const QModelIndexList& indexes)
 {
 	QSet<QModelIndex> alreadyIn;
-	return getAllIndexes(indexes, alreadyIn);
-}
-
-static bool modelIndexLessThan(const QModelIndex &i1, const QModelIndex &i2)
-{
-	QModelIndex p1(i1.parent());
-	QModelIndex p2(i2.parent());
-	if (i1 == p2) return true;
-	else if (p1 == p2) {
-		return (i1.row() < i2.row());
-	}
-	else if (p1.isValid() && modelIndexLessThan(p1, i2)) return true;
-	return false;
+	QModelIndexList ret(getAllIndexes(indexes, alreadyIn));
+	qStableSort(ret.begin(), ret.end(), modelIndexLessThan);
+	return ret;
 }
 
 QModelIndexList EntriesViewHelper::getAllIndexes(const QModelIndexList& indexes, QSet<QModelIndex>& alreadyIn)
@@ -275,7 +288,6 @@ QModelIndexList EntriesViewHelper::getAllIndexes(const QModelIndexList& indexes,
 			ret += getAllIndexes(childs, alreadyIn);
 		}
 	}
-	qStableSort(ret.begin(), ret.end(), modelIndexLessThan);
 	return ret;
 }
 
@@ -431,11 +443,15 @@ bool EntriesViewHelper::eventFilter(QObject *obj, QEvent *ev)
 				QMenu *menu(contextMenu());
 				if (menu->actions().isEmpty()) return true;
 				QContextMenuEvent *cev(static_cast<QContextMenuEvent *>(ev));
-				QList<EntryPointer> _selectedEntries(selectedEntries());
-				// This is stupid, but const-safety forces us here
-				QList<ConstEntryPointer> selectedEntries;
-				foreach (const EntryPointer &entry, _selectedEntries) selectedEntries << entry;
-				updateStatus(selectedEntries);
+				QList<ConstEntryPointer> cSelectedEntries;
+				if (client()->selectionModel()->selectedIndexes().size() < 250) {
+					QList<EntryPointer> _selectedEntries(selectedEntries());
+					// This is stupid, but const-safety forces us here
+					foreach (const EntryPointer &entry, _selectedEntries) cSelectedEntries << entry;
+					updateStatus(cSelectedEntries);
+				} else {
+					setEnabledAll(true);
+				}
 				menu->exec(client()->mapToGlobal(cev->pos()));
 				return false;
 			}
