@@ -273,15 +273,15 @@ void MainWindow::about()
 #error No version defined - the -DVERSION=<version> flag must be set!
 #endif
 	QString message = QString(
-		"<p>Copyright (C) 2008-2011 Alexandre Courbot.</p>"
-		"<p align=\"center\"><a href=\"http://www.tagaini.net\">http://www.tagaini.net</a></p><p>This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under the conditions of the <a href=\"http://www.gnu.org/licenses/gpl-3.0.html\">GNU General Public Licence, version 3.0</a>.</p><hr/>"
+		"<p>Copyright (C) 2008-2012 Alexandre Courbot.</p>"
+		"<p align=\"center\"><a href=\"http://www.tagaini.net\">http://www.tagaini.net</a></p><p>This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under the conditions of the <a href=\"http://www.gnu.org/licenses/gpl-3.0.html\">GNU General Public License, version 3.0</a>, or (at your option) any later version.</p><hr/>"
 		);
 	QString credits = "<p>Tagaini Jisho uses data from various sources:</p>";
 
 	foreach (const Plugin *plugin, Plugin::plugins()) {
 		credits += plugin->pluginInfo();
 	}
-	credits += "<p>Fugue Icons Copyright (C) 2010 <a href=\"http://p.yusukekamiyamane.com/\">Yusuke Kamiyamane</a>. All rights reserved. The icons are licensed under a <a href=\"http://creativecommons.org/licenses/by/3.0/\">Creative Commons Attribution 3.0 license</a>.</p>";
+	credits += "<p>Fugue Icons Copyright (C) 2010 <a href=\"http://p.yusukekamiyamane.com/\">Yusuke Kamiyamane</a>, licensed under a <a href=\"http://creativecommons.org/licenses/by/3.0/\">Creative Commons Attribution 3.0 license</a>.</p>";
 
 	Ui::AboutDialog aboutDialogUI;
 	QDialog aboutDialog;
@@ -298,8 +298,8 @@ void MainWindow::about()
 	QString authors =
 		"<h2>Authors</h2>"
 		"<p><b>Alexandre Courbot</b> project lead, programming.</p>"
+		"<p><b>Axel Bodart</b> CMake improvements, web site guru, Mac release manager.</p>"
 		"<p><b>Neil Caldwell</b> documentation.</p>"
-		"<p><b>Axel Bodart</b> project build, Mac release manager.</p>"
 		"<h2>Thanks to</h2>"
 		"<p><b>Philip Seyfi</b> application icon.</p>"
 		"<p><b>Tracy Poff</b> bug reports, documentation fixes.</p>"
@@ -327,7 +327,6 @@ void MainWindow::manual()
 #else
 	QString f = QDir(QCoreApplication::applicationDirPath()).filePath("doc/manual.html");
 #endif
-	qDebug() << f;
 	QDesktopServices::openUrl(f);
 }
 
@@ -356,7 +355,7 @@ void MainWindow::updateAvailable(const QString &version)
 	QPushButton laterButton(tr("Maybe later"));
 	messageBox.addButton(&laterButton, QMessageBox::RejectRole);
 	if (messageBox.exec() == QMessageBox::AcceptRole) {
-		QDesktopServices::openUrl(QUrl("http://www.tagaini.net"));
+		QDesktopServices::openUrl(QUrl("http://www.tagaini.net/download"));
 	}
 	_updateTimer.stop();
 }
@@ -371,7 +370,7 @@ void MainWindow::betaUpdateAvailable(const QString &version)
 	QPushButton laterButton(tr("Maybe later"));
 	messageBox.addButton(&laterButton, QMessageBox::RejectRole);
 	if (messageBox.exec() == QMessageBox::AcceptRole) {
-		QDesktopServices::openUrl(QUrl("http://www.tagaini.net"));
+		QDesktopServices::openUrl(QUrl("http://www.tagaini.net/download"));
 	}
 	_updateTimer.stop();
 }
