@@ -90,18 +90,23 @@ void PreferencesWindow::applySettings()
 
 static const QMap<QString, QString> _langMapping()
 {
-        QMap<QString, QString> ret;
-        ret.insert("en", QT_TRANSLATE_NOOP("GeneralPreferences", "English"));
-        ret.insert("fr", QT_TRANSLATE_NOOP("GeneralPreferences", "French"));
-        ret.insert("de", QT_TRANSLATE_NOOP("GeneralPreferences", "German"));
-        ret.insert("es", QT_TRANSLATE_NOOP("GeneralPreferences", "Spanish"));
-        ret.insert("ru", QT_TRANSLATE_NOOP("GeneralPreferences", "Russian"));
-        ret.insert("nl", QT_TRANSLATE_NOOP("GeneralPreferences", "Dutch"));
-        ret.insert("it", QT_TRANSLATE_NOOP("GeneralPreferences", "Italian"));
-        ret.insert("nb", QT_TRANSLATE_NOOP("GeneralPreferences", "Norvegian Bokmal"));
-        ret.insert("cs", QT_TRANSLATE_NOOP("GeneralPreferences", "Czech"));
-        ret.insert("ja", QT_TRANSLATE_NOOP("GeneralPreferences", "Japanese"));
-        ret.insert("pl", QT_TRANSLATE_NOOP("GeneralPreferences", "Polish"));
+	QMap<QString, QString> ret;
+	ret.insert("cs", QT_TRANSLATE_NOOP("GeneralPreferences", "Czech"));
+	ret.insert("de", QT_TRANSLATE_NOOP("GeneralPreferences", "German"));
+	ret.insert("en", QT_TRANSLATE_NOOP("GeneralPreferences", "English"));
+	ret.insert("es", QT_TRANSLATE_NOOP("GeneralPreferences", "Spanish"));
+	ret.insert("fr", QT_TRANSLATE_NOOP("GeneralPreferences", "French"));
+	ret.insert("it", QT_TRANSLATE_NOOP("GeneralPreferences", "Italian"));
+	ret.insert("ja", QT_TRANSLATE_NOOP("GeneralPreferences", "Japanese"));
+	ret.insert("nb", QT_TRANSLATE_NOOP("GeneralPreferences", "Norvegian Bokmal"));
+	ret.insert("nl", QT_TRANSLATE_NOOP("GeneralPreferences", "Dutch"));
+	ret.insert("pl", QT_TRANSLATE_NOOP("GeneralPreferences", "Polish"));
+	ret.insert("pt", QT_TRANSLATE_NOOP("GeneralPreferences", "Portuguese"));
+	ret.insert("ru", QT_TRANSLATE_NOOP("GeneralPreferences", "Russian"));
+	ret.insert("sv", QT_TRANSLATE_NOOP("GeneralPreferences", "Swedish"));
+	ret.insert("th", QT_TRANSLATE_NOOP("GeneralPreferences", "Thai"));
+	ret.insert("tr", QT_TRANSLATE_NOOP("GeneralPreferences", "Turkish"));
+	ret.insert("vi", QT_TRANSLATE_NOOP("GeneralPreferences", "Vietnamese"));
 	return ret;
 }
 
@@ -134,11 +139,18 @@ GeneralPreferences::GeneralPreferences(QWidget *parent) : PreferencesWindowCateg
 
 	QList<qps> prefLangs = _prefLangs();
 	foreach (const qps &langPair, prefLangs) {
-		QString lang = langPair.second;
+		QString lang(langPair.second);
 		const QString &enLang = langMapping[langPair.first];
+		QIcon langIcon(QString(":/images/flags/%1.png").arg(langPair.first));
 		if (lang != enLang) lang += QString(" (%1)").arg(enLang);
-		prefLanguage->addItem(lang);
-		prefLanguage->setItemData(prefLanguage->count() - 1, langPair.first);
+		if (langPair.first == "en" || Lang::supportedGUILanguages().contains(langPair.first)) {
+			guiLanguage->addItem(langIcon, lang);
+			guiLanguage->setItemData(guiLanguage->count() - 1, langPair.first);
+		}
+		if (langPair.first == "en" || Lang::supportedDictLanguages().contains(langPair.first)) {
+			dictLanguage->addItem(langIcon, lang);
+			dictLanguage->setItemData(dictLanguage->count() - 1, langPair.first);
+		}
 	}
 
 	firstDayOfWeek->addItem(tr("Monday"), Qt::Monday);
@@ -155,13 +167,18 @@ void GeneralPreferences::refresh()
 	fontChooser->setDefault(MainWindow::applicationFont.isDefault());
 	fontChooser->setFont(QFont());
 
-	int idx;
-	QList<qps> prefLangs = _prefLangs();
-	for (idx = 0; idx < prefLangs.size(); idx++) {
-		if (prefLangs[idx].first == Lang::preferredLanguage.value()) break;
-	}
-	if (idx >= prefLangs.size()) idx = -1;
-	prefLanguage->setCurrentIndex(idx + 1);
+	for (int idx = 1; idx < guiLanguage->count(); idx++)
+		if (guiLanguage->itemData(idx) == Lang::preferredGUILanguage.value()) {
+			guiLanguage->setCurrentIndex(idx);
+			break;
+		}
+	for (int idx = 1; idx < dictLanguage->count(); idx++)
+		if (dictLanguage->itemData(idx) == Lang::preferredDictLanguage.value()) {
+			dictLanguage->setCurrentIndex(idx);
+			break;
+		}
+
+	alwaysShowEnglish->setChecked(Lang::alwaysShowEnglish.value());
 
 	firstDayOfWeek->setCurrentIndex(firstDayOfWeek->findData(RelativeDate::firstDayOfWeek.value()));
 
@@ -215,8 +232,13 @@ void GeneralPreferences::applySettings()
 	else MainWindow::applicationFont.set(font.toString());
 
 	// Application language
-	if (prefLanguage->currentIndex() == 0) Lang::preferredLanguage.reset();
-	else Lang::preferredLanguage.set(prefLanguage->itemData(prefLanguage->currentIndex()).toString());
+	if (guiLanguage->currentIndex() == 0) Lang::preferredGUILanguage.reset();
+	else Lang::preferredGUILanguage.set(guiLanguage->itemData(guiLanguage->currentIndex()).toString());
+	if (dictLanguage->currentIndex() == 0) Lang::preferredDictLanguage.reset();
+	else Lang::preferredDictLanguage.set(dictLanguage->itemData(dictLanguage->currentIndex()).toString());
+
+	// Always show English
+	Lang::alwaysShowEnglish.set(alwaysShowEnglish->isChecked());
 
 	// First day of week
 	RelativeDate::firstDayOfWeek.set(static_cast<Qt::DayOfWeek>(firstDayOfWeek->itemData(firstDayOfWeek->currentIndex()).toInt()));
