@@ -337,7 +337,7 @@ void Kanjidic2EntryFormatter::showToolTip(const ConstKanjidic2EntryPointer entry
 		CLOSE_TAB;
 	}
 	if (entry->jlpt() != -1 && tooltipShowJLPT.value()) {
-		QString body(tr("<b>JLPT:</b> N%1").arg(entry->jlpt() < 3 ? entry->jlpt() : entry->jlpt() + 1));
+		QString body(tr("<b>JLPT:</b> N%1").arg(entry->jlpt()));
 		CLOSE_TAB;
 	}
 	if (entry->heisig() != -1 && tooltipShowHeisig.value()) {
@@ -392,7 +392,7 @@ QString Kanjidic2EntryFormatter::shortDesc(const ConstKanjidic2EntryPointer &sha
 	ret += autoFormat(str);
 	
 	if (shortDescShowJLPT.value() && entry->jlpt() != -1)
-		ret += QString(" <b>%1</b>").arg(autoFormat(tr("(JLPT N%1)").arg(entry->jlpt() < 3 ? entry->jlpt() : entry->jlpt() + 1)));
+		ret += QString(" <b>%1</b>").arg(autoFormat(tr("(JLPT N%1)").arg(entry->jlpt())));
 	ret += QString(" <a href=\"entry://?type=%1&id=%2\"><img src=\"moreicon\"/></a>").arg(entry->type()).arg(entry->id());
 	if (entry->trained()) {
 		ret = QString("<span style=\"background-color:%1\">%2</span>").arg(colorTriplet(entry->scoreColor())).arg(ret);
@@ -422,7 +422,7 @@ QString Kanjidic2EntryFormatter::formatMeanings(const ConstEntryPointer &_entry)
 		}
 		means[mean.lang()] << mean.meaning();
 	}
-	foreach (const QString &lang, Lang::preferredLanguages()) {
+	foreach (const QString &lang, Lang::preferredDictLanguages()) {
 		if (!means.contains(lang)) continue;
 		meanStrings << QString("<img src=\"flag:%1\"/> ").arg(lang) + means[lang].join(", ");
 	}
@@ -494,7 +494,7 @@ QString Kanjidic2EntryFormatter::formatJLPT(const ConstEntryPointer &_entry) con
 {
 	ConstKanjidic2EntryPointer entry(_entry.staticCast<const Kanjidic2Entry>());
 	if (entry->jlpt() != -1 && showJLPT.value()) {
-		return QString("<b>%1:</b> N%2").arg(tr("JLPT")).arg(entry->jlpt() < 3 ? entry->jlpt() : entry->jlpt() + 1);
+		return QString("<b>%1:</b> N%2").arg(tr("JLPT")).arg(entry->jlpt());
 	}
 	return "";
 }
@@ -648,7 +648,7 @@ void ShowUsedInKanjiJob::result(EntryPointer entry)
 void ShowUsedInKanjiJob::completed()
 {
 	if (!gotResults) return;
-	cursor().insertHtml(QString("<a href=\"component:?reset=true&kanji=%1\" title=\"%4\">%2</a> <a href=\"component:?kanji=%1\" title=\"%5\">%3</a>").arg(_kanji).arg(tr("All compounds")).arg(tr("(+)")).arg("Make a new search using only this filter").arg("Add this filter to the current search"));
+	cursor().insertHtml(QString("<a href=\"component:?reset=true&kanji=%1\" title=\"%4\">%2</a> <a href=\"component:?kanji=%1\" title=\"%5\">%3</a>").arg(_kanji).arg(tr("All compounds")).arg(tr("(+)")).arg(tr("Make a new search using only this filter")).arg(tr("Add this filter to the current search")));
 }
 
 ShowUsedInWordsJob::ShowUsedInWordsJob(const QString &kanji, const QTextCursor &cursor) :
@@ -674,5 +674,5 @@ void ShowUsedInWordsJob::result(EntryPointer entry)
 void ShowUsedInWordsJob::completed()
 {
 	if (!gotResults) return;
-	cursor().insertHtml(QString("<a href=\"allwords:?reset=true&kanji=%1\" title=\"%4\">%2</a> <a href=\"allwords:?kanji=%1\" title=\"%5\">%3</a>").arg(_kanji).arg(tr("All words using this kanji")).arg(tr("(+)")).arg("Make a new search using only this filter").arg("Add this filter to the current search"));
+	cursor().insertHtml(QString("<a href=\"allwords:?reset=true&kanji=%1\" title=\"%4\">%2</a> <a href=\"allwords:?kanji=%1\" title=\"%5\">%3</a>").arg(_kanji).arg(tr("All words using this kanji")).arg(tr("(+)")).arg(tr("Make a new search using only this filter")).arg(tr("Add this filter to the current search")));
 }
